'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.DataEntity = exports.States = exports.DELETE_MANY = exports.UPDATE_MANY = exports.READ_MANY = exports.CREATE_MANY = exports.DELETE_ONE = exports.UPDATE_ONE = exports.READ_ONE = exports.CREATE_ONE = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _immutable = require('immutable');

var _uniqid = require('uniqid');

var _uniqid2 = _interopRequireDefault(_uniqid);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var PREFIX = 'RDE';
var Actions = {
  CREATE_ONE: 'CREATE_ONE',
  READ_ONE: 'READ_ONE',
  UPDATE_ONE: 'UPDATE_ONE',
  DELETE_ONE: 'DELETE_ONE',
  CREATE_MANY: 'CREATE_MANY',
  READ_MANY: 'READ_MANY',
  UPDATE_MANY: 'UPDATE_MANY',
  DELETE_MANY: 'DELETE_MANY'
};
var CREATE_ONE = Actions.CREATE_ONE,
    READ_ONE = Actions.READ_ONE,
    UPDATE_ONE = Actions.UPDATE_ONE,
    DELETE_ONE = Actions.DELETE_ONE,
    CREATE_MANY = Actions.CREATE_MANY,
    READ_MANY = Actions.READ_MANY,
    UPDATE_MANY = Actions.UPDATE_MANY,
    DELETE_MANY = Actions.DELETE_MANY;
exports.CREATE_ONE = CREATE_ONE;
exports.READ_ONE = READ_ONE;
exports.UPDATE_ONE = UPDATE_ONE;
exports.DELETE_ONE = DELETE_ONE;
exports.CREATE_MANY = CREATE_MANY;
exports.READ_MANY = READ_MANY;
exports.UPDATE_MANY = UPDATE_MANY;
exports.DELETE_MANY = DELETE_MANY;
var States = exports.States = {
  START: 'START',
  SUCCESS: 'SUCCESS',
  FAIL: 'FAIL'
};

var defaultConfig = {
  keyExtractor: function keyExtractor(unit) {
    return unit.get('id');
  },
  keyGenerator: function keyGenerator() {
    return _uniqid2.default.time();
  },
  cacheRequestsCount: 3,
  cacheValidityTime: null
};

var defaultActionConfig = {
  optimistic: false,
  force: false,
  data: {},
  params: {}
};

var DataEntity = exports.DataEntity = function () {
  function DataEntity(config) {
    _classCallCheck(this, DataEntity);

    this._lastRequestIndex = 0;

    this.config = _extends({}, defaultConfig, config);
    this.clear();
  }

  _createClass(DataEntity, [{
    key: 'perform',
    value: function perform(dispatch) {
      var _this = this;

      return function (action, config, meta) {
        config = _extends({}, defaultActionConfig, config);
        if (!_this.isPerforming(action, config) || config.force) {
          var actionPromise = _this.config.process(action, config);
          var requestId = ++_this._lastRequestIndex;
          dispatch({ type: _this.getConst(action, States.START), config: config, requestId: requestId, meta: meta });
          if (actionPromise !== null) {
            actionPromise.then(function (data) {
              dispatch({ type: _this.getConst(action, States.SUCCESS), payload: data, requestId: requestId, meta: meta });
            }, function (error) {
              dispatch({ type: _this.getConst(action, States.FAIL), payload: error, requestId: requestId, meta: meta });
            });
          }
          return requestId;
        }
        return _this._requestStates.get(action).findKey(function (request) {
          return request.get('active') && request.get('config').equals(config);
        });
      };
    }
  }, {
    key: 'clear',
    value: function clear() {
      this._previousState = null;
      this._requestStates = new _immutable.Map(Object.keys(Actions).reduce(function (mapped, key) {
        return _extends({}, mapped, _defineProperty({}, Actions[key], new _immutable.OrderedMap()));
      }, {}));
    }
  }, {
    key: 'isPerforming',
    value: function isPerforming(action, config) {
      var actionState = this._requestStates.get(action).filter(function (request) {
        return request.get('active');
      });
      if (!config) {
        return actionState.size > 0;
      }
      return typeof actionState.find(function (request) {
        return request.get('config').equals(config);
      }) !== 'undefined';
    }
  }, {
    key: 'sortFinishedRequests',
    value: function sortFinishedRequests(requests) {
      return requests.filter(function (action, key, request) {
        return !request.getIn([key, 'active']);
      }).sortBy(function (value) {
        return value.get('finish');
      }, function (timeA, timeB) {
        return timeA < timeB ? -1 : +(timeA > timeB);
      });
    }
  }, {
    key: 'getLastError',
    value: function getLastError(action, config) {
      var actionState = this._requestStates.get(action);
      if (config) {
        var requestState = actionState.find(function (request) {
          return request.get('config').equals(config);
        });
        return typeof requestState !== 'undefined' && requestState.has('error') ? new Error(requestState.get('error')) : null;
      }
      var lastError = this.sortFinishedRequests(actionState).findLast(function (request) {
        return request.has('error');
      });
      return typeof lastError !== 'undefined' ? new Error(lastError.get('error')) : null;
    }
  }, {
    key: 'getReducers',
    value: function getReducers() {
      var _this2 = this,
          _ref2;

      return _ref2 = {}, _defineProperty(_ref2, 'internal(RDE:' + this.config.reducerName + ')', function undefined(oldState, _ref) {
        var type = _ref.type,
            config = _ref.config,
            requestId = _ref.requestId,
            payload = _ref.payload;

        var action = _this2.parseAction(type);
        var state = _this2.parseState(type);
        if (action !== null && state !== null) {
          var requestState = {};
          if (~[States.FAIL, States.SUCCESS].indexOf(state)) {
            if (!_this2._requestStates.hasIn([action, requestId])) {
              return _this2._requestStates;
            }
            requestState.active = false;
            requestState[state === States.FAIL ? 'error' : 'payload'] = payload;
            requestState.finish = Date.now();
          } else {
            requestState.config = config;
            requestState.active = true;
            requestState.start = Date.now();
          }
          requestState.action = action;
          _this2._newState(_this2._requestStates.mergeIn([action, requestId], (0, _immutable.fromJS)(requestState)));
        }
        return _this2._requestStates;
      }), _defineProperty(_ref2, this.config.reducerName, function (state) {
        if (_this2._previousState !== null && (0, _immutable.is)(_this2._previousState, _this2._requestStates)) {
          return state || new _immutable.OrderedMap();
        }
        var reducerState = new _immutable.OrderedMap();
        reducerState = reducerState.withMutations(function (orderedMap) {
          if (_this2.config.reducerDefault) {
            _this2.config.reducerDefault.forEach(function (item) {
              var key = _this2.config.keyExtractor(item).toString();
              orderedMap.set(key, item);
            });
          }
          _this2.sortFinishedRequests(_this2._requestStates.flatten(1)).forEach(function (request) {
            var payload = request.get('payload', null);
            switch (request.get('action')) {
              case READ_ONE:
              case CREATE_ONE:
              case UPDATE_ONE:
                if (payload !== null) {
                  payload = new _immutable.List([payload]);
                }
              case READ_MANY:
              case CREATE_MANY:
              case UPDATE_MANY:
                if (payload !== null && _immutable.Iterable.isIterable(payload)) {
                  payload.forEach(function (item) {
                    var key = _this2.config.keyExtractor(item).toString();
                    orderedMap.set(key, item);
                  });
                }
                break;
              case DELETE_ONE:
              case DELETE_MANY:
                if (!request.has('error')) {
                  request.getIn(['config', 'keys'], new _immutable.List()).forEach(function (key) {
                    orderedMap.delete(key);
                  });
                }
                break;
            }
          });
          _this2._requestStates.forEach(function (requests, action) {
            requests.forEach(function (request) {
              if (request.getIn(['config', 'optimistic'], false) && request.get('active', false)) {
                var data = request.getIn(['config', 'data'], new _immutable.List());
                var actionKeys = request.getIn(['config', 'keys']);
                switch (action) {
                  case CREATE_ONE:
                    data = new _immutable.List([data]);
                  case CREATE_MANY:
                    if (_immutable.Iterable.isIterable(data)) {
                      data.forEach(function (item) {
                        var key = _this2.config.keyExtractor(item) || _this2.config.keyGenerator(item);
                        orderedMap.set(key, item);
                      });
                    }
                    break;
                  case UPDATE_ONE:
                    data = new _immutable.List([data]);
                  case UPDATE_MANY:
                    if (_immutable.Iterable.isIterable(data)) {
                      data.forEach(function (item, index) {
                        var key = actionKeys && actionKeys.get(index) || _this2.config.keyExtractor(item);
                        orderedMap.mergeDeepIn([key.toString()], item);
                      });
                    }
                    break;
                  case DELETE_ONE:
                  case DELETE_MANY:
                    if (_immutable.Iterable.isIterable(actionKeys)) {
                      actionKeys.forEach(function (key) {
                        orderedMap.delete(key.toString());
                      });
                    }
                    break;
                }
              }
            });
          });
        });
        _this2._previousState = _this2._requestStates;
        return reducerState;
      }), _ref2;
    }
  }, {
    key: 'getConst',
    value: function getConst(action, state) {
      return PREFIX + '/' + this.config.reducerName + '/' + action + '_' + state;
    }
  }, {
    key: '_newState',
    value: function _newState(state) {
      var _this3 = this;

      this._requestStates = state.map(function (requests, action) {
        return requests.takeLast(_this3.config.cacheRequestsCount);
      });
    }
  }, {
    key: 'parseAction',
    value: function parseAction(actionType) {
      for (var action in Actions) {
        if (actionType.startsWith(PREFIX + '/' + this.config.reducerName + '/' + action)) {
          return action;
        }
      }
      return null;
    }
  }, {
    key: 'parseState',
    value: function parseState(actionType) {
      var action = this.parseAction(actionType);
      if (action !== null) {
        for (var state in States) {
          if (actionType === PREFIX + '/' + this.config.reducerName + '/' + action + '_' + state) {
            return state;
          }
        }
      }
      return null;
    }
  }]);

  return DataEntity;
}();